from autobreadcrumbs import site
from django.utils.translation import ugettext_lazy

site.update({
    'project-index': ugettext_lazy('PO Projects'),
    'project-create': ugettext_lazy('Create a new project'),
    'project-details': ugettext_lazy('{% load i18n %}<small>{% trans "Project" %}</small> {{ project.name }}'),
    'project-update': ugettext_lazy('Project update'),
    'project-download': None,
    'catalog-details': ugettext_lazy('{% load i18n %}<small>{% trans "Catalog" %}</small> {{ catalog.get_locale_name }}'),
    'catalog-messages-edit': ugettext_lazy('Edit messages'),
    'catalog-messages-download': None,
})


# Import from django
from django.contrib import admin
#from guardian.admin import GuardedModelAdmin

# Import from here
from .models import Project, ProjectVersion, TemplateMsg, Catalog, TranslationMsg

    
#class ProjectAdmin(GuardedModelAdmin):
class ProjectAdmin(admin.ModelAdmin):
    ordering = ('name',)
    search_fields = ('slug', 'name',)
    list_display = ('slug', 'name')
    prepopulated_fields = {"slug": ("name",)}
    
class ProjectVersionAdmin(admin.ModelAdmin):
    ordering = ('project', 'version',)
    list_filter = ('project',)
    #list_display = ('version', 'project',)
    fieldsets = (
        (None, {
            'fields': ('project', 'version')
        }),
        ('Metas', {
            'classes': ('collapse',),
            'fields': ('header_comment', 'mime_headers')
        }),
    )
    
class CatalogAdmin(admin.ModelAdmin):
    ordering = ('project_version', 'locale',)
    list_filter = ('project_version',)
    list_display = ('project_version', 'locale',)
    list_display_links = ('project_version', 'locale',)
    raw_id_fields = ("project_version",)
    fieldsets = (
        (None, {
            'fields': ('project_version', 'locale')
        }),
        ('Metas', {
            'classes': ('collapse',),
            'fields': ('header_comment', 'mime_headers')
        }),
    )
    
class TemplateMsgAdmin(admin.ModelAdmin):
    ordering = ('project_version', 'message',)
    list_filter = ('project_version',)
    list_display = ('project_version', 'get_truncated_message',)
    raw_id_fields = ("project_version",)
    search_fields = ('message',)
    fieldsets = (
        (None, {
            'fields': ('project_version', 'message')
        }),
        ('Metas', {
            'classes': ('collapse',),
            'fields': ('locations', 'flags')
        }),
    )
    
class TranslationMsgAdmin(admin.ModelAdmin):
    ordering = ('catalog', 'message',)
    list_filter = ('catalog',)
    list_display = ('catalog', 'fuzzy', 'get_truncated_message',)
    list_display_links = ('catalog', 'get_truncated_message',)
    raw_id_fields = ("template","catalog",)
    search_fields = ('message',)
    fieldsets = (
        (None, {
            'fields': ('template', 'catalog')
        }),
        ('Content', {
            'fields': ('message', 'fuzzy')
        }),
        ('Options', {
            'fields': ('pluralizable', 'python_format')
        }),
    )


admin.site.register(Project, ProjectAdmin)
admin.site.register(ProjectVersion, ProjectVersionAdmin)
admin.site.register(TemplateMsg, TemplateMsgAdmin)
admin.site.register(Catalog, CatalogAdmin)
admin.site.register(TranslationMsg, TranslationMsgAdmin)



"""
REST API

TODO: Include the following doc to the README

Install `djangorestframework`_

Add this to your settings : ::

    INSTALLED_APPS = (
        ...
        'rest_framework'
        ...
    )

    REST_FRAMEWORK = {
        'PAGINATE_BY': 10,
        # Use hyperlinked styles by default.
        # Only used if the `serializer_class` attribute is not set on a view.
        'DEFAULT_MODEL_SERIALIZER_CLASS': (
            'rest_framework.serializers.HyperlinkedModelSerializer',
        ),

        # Use Django's standard `django.contrib.auth` permissions,
        # or allow read-only access for unauthenticated users.
        'DEFAULT_PERMISSION_CLASSES': (
            'rest_framework.permissions.IsAdminUser',
            #'rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly',
        ),
    }
    
And this to **your project's urls.py** : ::

    urlpatterns = patterns('',
        ...
        (r'^api-auth/', include('rest_framework.urls', namespace='rest_framework')),
        ...
    )

Then the API is reachable from ``http://yoururl/po/rest/`` that will display available endpoints. You must be authenticated to access to the API.

Actually this will serve some endpoints :

* Project list
* Project detail to retrieve or update (TODO) a Project instance
* Project tarball (but for the last version only)
"""



from cStringIO import StringIO

from django.conf import settings
from django.http import Http404
from django.shortcuts import get_object_or_404

from rest_framework.views import APIView
from rest_framework import generics
from rest_framework.response import Response
from rest_framework import status, viewsets, generics, renderers
from rest_framework.decorators import api_view
from rest_framework.reverse import reverse

from po_projects.models import Project, ProjectVersion, TemplateMsg, Catalog, TranslationMsg
from po_projects.rest.serializers import ProjectCurrentSerializer, ProjectVersionSerializer
from po_projects.mixins import DownloadMixin
from po_projects.dump import po_project_export

@api_view(('GET',))
def api_root(request, format=None):
    """
    This is the entry point that display available endpoints for this API
    """
    return Response({
        'projects': reverse('po_projects:api-project-list', request=request, format=format)
    })


class ProjectList(APIView):
    """
    List all available projects
    """
    def get(self, request, format=None):
        projects = Project.objects.all()
        serializer = ProjectCurrentSerializer(projects, many=True, context={'request': request})

        return Response(serializer.data)


class ProjectDetailMixin(object):
    """
    Mixin to get Project object
    
    NOTE: is this required anymore ?
    """
    def get_object(self):
        return get_object_or_404(Project, slug=self.kwargs['slug'])


class ProjectCurrentDetail(ProjectDetailMixin, generics.RetrieveUpdateAPIView):
    """
    Retrieve and update the current last version of a project instance from its slug. 
    
    * ``projectversion_set`` attribute will contains the project versions;
    * ``tarball_url`` attribute is a link to download a ZIP archive of PO files for the 
      current project version;
    
    ``projectversion_set`` urls use the version ID (pk), not the slug or version name.
    
    This view is a "Retrieve and update" view, so you can see and get project details 
    but also update its content.
    
    The ``pot`` field is optionnal, you can fill it with the content of a valid POT file 
    to update project template catalog and its translation catalogs.
    """
    serializer_class = ProjectCurrentSerializer
    model = Project


class ProjectVersionDetail(APIView):
    """
    Retrieve a project instance for a specific version given by its ID.
    
    Project versions are freezed, so you can't update them.
    """
    def get_object(self, pk):
        return get_object_or_404(ProjectVersion, pk=pk)

    def get(self, request, pk, format=None):
        self.project_version = self.get_object(pk)
        self.object = self.project_version.project
        serializer = ProjectVersionSerializer(self.object, context={'request': request})
        return Response(serializer.data)


class ProjectArchive(ProjectDetailMixin, DownloadMixin, APIView):
    """
    Send a ZIP archive for the project's PO files
    
    TODO: share code from a mixin using code from ProjectExportView
    """
    content_type = 'application/x-gzip'
    filename_format = "{project_slug}_{timestamp}.tar.gz"
    
    def get(self, request, slug, format=None):
        self.object = self.get_object()
        return super(ProjectArchive, self).get(request)

    def get_catalog_kind(self):
        kind = self.request.GET.get('kind', settings.DEFAULT_CATALOG_FILENAMES)
        if kind not in settings.AVAILABLE_CATALOG_FILENAMES:
            return settings.DEFAULT_CATALOG_FILENAMES
        return kind

    def get_context_data(self, **kwargs):
        context = super(ProjectArchive, self).get_context_data(**kwargs)
        context.update({
            'project': self.object,
            'project_slug': self.object.slug,
            'timestamp': self.get_filename_timestamp(),
        })
        return context
    
    def get_filename(self, context):
        return self.filename_format.format(**context)
    
    def get_content(self, context):
        archive_file = StringIO()
        
        po_project_export(self.object, self.object.get_current_version(), archive_file, catalog_filename=self.get_catalog_kind())
        
        return archive_file.getvalue()



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Project.description'
        db.add_column(u'po_projects_project', 'description',
                      self.gf('django.db.models.fields.TextField')(default='', blank=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Project.description'
        db.delete_column(u'po_projects_project', 'description')


    models = {
        u'po_projects.catalog': {
            'Meta': {'object_name': 'Catalog'},
            'header_comment': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'locale': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'mime_headers': ('django.db.models.fields.TextField', [], {}),
            'project_version': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['po_projects.ProjectVersion']"})
        },
        u'po_projects.project': {
            'Meta': {'object_name': 'Project'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '75'})
        },
        u'po_projects.projectversion': {
            'Meta': {'object_name': 'ProjectVersion'},
            'header_comment': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mime_headers': ('django.db.models.fields.TextField', [], {}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['po_projects.Project']"}),
            'version': ('django.db.models.fields.SmallIntegerField', [], {})
        },
        u'po_projects.templatemsg': {
            'Meta': {'object_name': 'TemplateMsg'},
            'flags': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'locations': ('django.db.models.fields.TextField', [], {}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'project_version': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['po_projects.ProjectVersion']"})
        },
        u'po_projects.translationmsg': {
            'Meta': {'object_name': 'TranslationMsg'},
            'catalog': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['po_projects.Catalog']"}),
            'fuzzy': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'pluralizable': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'python_format': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'template': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['po_projects.TemplateMsg']"})
        }
    }

    complete_apps = ['po_projects']


import json

from po_projects.models import TemplateMsg, TranslationMsg

def create_templatemsgs(project_version, pot_catalog, commit=True):
    """
    Create template messages from a POT catalog
    
    @project_version: ProjectVersion instance
    @pot_catalog: Babel Catalog instance
    """
    entries = []
    for message in pot_catalog:
        if message.id:
            #print message.id
            flags = json.dumps(list(message.flags))
            locations = json.dumps(message.locations)
            entries.append(TemplateMsg(project_version=project_version, message=message.id, locations=locations, flags=flags))
        
    if commit:
        TemplateMsg.objects.bulk_create(entries)
        
    return entries

def create_new_version(project, version, uploaded_catalog):
    """
    Open a new version for a project and fill it with datas from the given 
    POT file
    
    @project: Project instance
    @version: Version label as a string
    @uploaded_catalog: Babel Catalog instance
    """
    project_version = project.projectversion_set.create(
        version = version,
        header_comment = uploaded_catalog.header_comment,
        mime_headers = json.dumps(dict(uploaded_catalog.mime_headers)),
    )
    create_templatemsgs(project_version, uploaded_catalog)
    
    return project_version

def update_catalogs(project, previous_version, current_version):
    """
    Recreate all existing catalogs from previous version in the current one, 
    then update them from the given POT file
    
    @project: Project instance
    @previous_version: ProjectVersion instance for the previous version
    @current_version: ProjectVersion instance for the newly created version
    """
    current_template = current_version.get_babel_template()
    current_templatemsg_map = dict([(item.message, item) for item in current_version.templatemsg_set.all()])
    
    # For each existing catalog in previous project version
    for previous_catalog in previous_version.catalog_set.all():
        # Update previous catalog from the given POT file
        current_babel_catalog = previous_catalog.get_babel_catalog()
        current_babel_catalog.update(current_template)
        
        # New catalog for current version
        current_catalog = current_version.catalog_set.create(
            locale = previous_catalog.locale,
            header_comment = current_version.header_comment,
            mime_headers = current_version.mime_headers,
        )
        
        # Add entries to the new catalog from template messages
        entries = []
        for template_id,template_instance in current_templatemsg_map.items():
            message = ''
            fuzzy = False
            if template_id in current_babel_catalog:
                if current_babel_catalog[template_id].string:
                    message = current_babel_catalog[template_id].string
                fuzzy = current_babel_catalog[template_id].fuzzy
            entries.append(TranslationMsg(template=template_instance, catalog=current_catalog, message=message, fuzzy=fuzzy))
        
        # Bulk saving entries
        TranslationMsg.objects.bulk_create(entries)



# -*- coding: utf-8 -*-
"""
Forms for po_projects
"""
import json

from django.conf import settings
from django import forms
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from crispy_forms.helper import FormHelper
from crispy_forms.bootstrap import UneditableField
from crispy_forms_foundation.layout import Layout, Fieldset, Row, Column, ButtonHolder, Submit

from babel import Locale
from babel.core import UnknownLocaleError, get_locale_identifier
from babel.messages.pofile import read_po

from po_projects.models import Project, ProjectVersion, TemplateMsg, Catalog, TranslationMsg

from po_projects.utils import create_templatemsgs, create_new_version, update_catalogs

class SourceTextField(UneditableField):
    """
    Layout object for rendering template field as simple html text
    """
    template = "po_projects/input_as_text.html"


class ProjectForm(forms.ModelForm):
    """Project Form"""
    po_file = forms.FileField(label=_('POT File'), required=True, help_text='Upload a valid POT file to initialize project strings to translate')
    
    def __init__(self, author=None, *args, **kwargs):
        self.author = author
        self.uploaded_catalog = None
        self.catalog_entries = []
        
        self.helper = FormHelper()
        self.helper.form_action = '.'
        self.helper.add_input(Submit('submit', 'Submit'))

        super(ProjectForm, self).__init__(*args, **kwargs)

    def clean_po_file(self):
        data = self.cleaned_data['po_file']
        if data:
            try:
                self.uploaded_catalog = read_po(data, ignore_obsolete=True)
            except:
                raise forms.ValidationError("Your file does not seem to be a valid POT file")

        return data

    def save(self, commit=True):
        project = super(ProjectForm, self).save(commit=False)
        project.save()
        
        # Create a the first project_version
        create_new_version(project, 1, self.uploaded_catalog)
            
        return project

    class Meta:
        model = Project


class ProjectUpdateForm(ProjectForm):
    """Project Form for update"""
    def __init__(self, author=None, *args, **kwargs):
        self.author = author
        super(ProjectUpdateForm, self).__init__(author, *args, **kwargs)
        
        self.fields['po_file'].required = False

    def save(self, commit=True):
        project = super(ProjectForm, self).save(commit=False)
        
        if commit:
            if self.uploaded_catalog:
                previous_version = project.get_current_version()
                current_version = create_new_version(project, previous_version.version+1, self.uploaded_catalog)
                update_catalogs(project, previous_version, current_version)
            project.save()
            
        return project


class CatalogForm(forms.ModelForm):
    """Catalog base Form"""
    def __init__(self, author=None, project_version=None, *args, **kwargs):
        self.author = author
        self.project_version = project_version
        self.fill_messages = not(kwargs.get('instance'))
        self.helper = FormHelper()
        self.helper.form_action = '.'
        self.helper.add_input(Submit('submit', 'Submit'))

        super(CatalogForm, self).__init__(*args, **kwargs)

    def clean_locale(self):
        data = self.cleaned_data['locale']
        if data:
            # only accept "_" as separator, all "-" are replaced to "_"
            data = "_".join(data.split("-"))
            try:
                self.locale_trans = Locale.parse(data)
            except UnknownLocaleError:
                raise forms.ValidationError("Invalid locale")
            except ValueError:
                raise forms.ValidationError("Invalid locale")
            else:
                data = get_locale_identifier((self.locale_trans.language, self.locale_trans.territory, self.locale_trans.script, self.locale_trans.variant), sep='_')
        return data

    def clean(self):
        cleaned_data = super(CatalogForm, self).clean()
        
        if not self.author.has_perm('po_projects.add_catalog'):
            raise forms.ValidationError(_("You don't have permission to use this form"))

        # Always return the full collection of cleaned data.
        return cleaned_data
    
    def save(self, commit=True):
        catalog = super(CatalogForm, self).save(commit=False)
        catalog.project_version = self.project_version
        
        if commit:
            catalog.header_comment = self.project_version.header_comment
            catalog.mime_headers = self.project_version.mime_headers
            catalog.save()
            
            # Only fill catalog with template messages on the first create
            if self.fill_messages:
                entries = []
                for row in self.project_version.templatemsg_set.all():
                    entries.append(TranslationMsg(template=row, catalog=catalog, message=''))
            
                TranslationMsg.objects.bulk_create(entries)
            
        return catalog

    class Meta:
        model = Catalog
        exclude = ('project_version', 'header_comment', 'mime_headers')


class CatalogUpdateForm(CatalogForm):
    """Catalog update Form"""
    po_file = forms.FileField(label=_('PO File'), required=False, help_text='Upload a valid PO file to update catalog messages, it will only update allready existing messages from the template, it does not add new message or remove existing messages. Be careful this will overwrite previous translations.')
    
    def __init__(self, author=None, project_version=None, *args, **kwargs):
        self.author = author
        self.fill_messages = False # Never re-fill catalog with translation from template
        self.uploaded_catalog = None
        
        super(CatalogUpdateForm, self).__init__(author, project_version, *args, **kwargs)

    def clean(self):
        cleaned_data = super(CatalogForm, self).clean()
        
        if not self.author.has_perm('po_projects.change_catalog'):
            raise forms.ValidationError(_("You don't have permission to use this form"))

        # Always return the full collection of cleaned data.
        return cleaned_data

    def clean_po_file(self):
        data = self.cleaned_data['po_file']
        if data:
            try:
                self.uploaded_catalog = read_po(data, ignore_obsolete=True)
            except:
                raise forms.ValidationError("Your file does not seem to be a valid PO file")

        return data

    def save(self, commit=True):
        catalog = super(CatalogUpdateForm, self).save(commit=commit)
        
        # Get all allready saved messages
        current_messages = dict([(item.template.message, item) for item in catalog.translationmsg_set.select_related('template').all()])
        
        if self.uploaded_catalog:
            uploaded_entries = []
            for message in self.uploaded_catalog:
                if message.id:
                    # Update message if allready exist in database and if different from the source
                    if message.id in current_messages and message.string != current_messages[message.id].message:
                        current_messages[message.id].message = message.string
                        current_messages[message.id].fuzzy = message.fuzzy
                        current_messages[message.id].pluralizable = message.pluralizable
                        current_messages[message.id].python_format = message.python_format
                        current_messages[message.id].save()
        
        return catalog


class TranslationMsgForm(forms.ModelForm):
    """Translation Form"""
    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_action = '.'
        self.helper.layout = Layout(
            Row(
                Column(
                    SourceTextField('template'),
                    css_class='small-6'
                ),
                Column(
                    'fuzzy',
                    'message',
                    css_class='small-6'
                ),
            ),
        )
        self.helper.add_input(Submit('submit', 'Submit'))

        super(TranslationMsgForm, self).__init__(*args, **kwargs)

    def save(self, commit=True):
        message = super(TranslationMsgForm, self).save(commit=False)
        
        if commit:
            message.save()
            
        return message

    class Meta:
        model = TranslationMsg
        exclude = ('catalog',)
        widgets = {
            'message': forms.Textarea(attrs={'rows': 3}),
        }




# -*- coding: utf-8 -*-
"""
Models for po_projects


> Project
    |
    |_> ProjectVersion
        |
        |_> TemplateMsg
        |
        |_> Catalog
            |
            |___> TranslationMsg

"""
import json

from babel.core import Locale
from babel.messages.catalog import Catalog as BabelCatalog

from django.db import models
from django.utils.translation import ugettext_lazy as _

class Project(models.Model):
    """
    Project container
    """
    name = models.CharField(_('name'), max_length=150)
    slug = models.SlugField(_('slug'), unique=True, max_length=75)
    description = models.TextField(_('description'), blank=True)

    def __unicode__(self):
        return self.name

    def get_current_version(self):
        return self.projectversion_set.all().order_by('-version')[0:1][0]

    class Meta:
        verbose_name = _('project')
        verbose_name_plural = _('projects')

class ProjectVersion(models.Model):
    """
    Project version is what contains catalogs and template catalogs for a specific POT file version
    """
    project = models.ForeignKey(Project, verbose_name=_('project'), blank=False)
    version = models.SmallIntegerField(_('version'))
    header_comment = models.TextField(_('header comment'))
    mime_headers = models.TextField(_('mime headers'))

    def get_babel_template(self):
        """
        Return a babel template catalog suitable for a POT file
        """
        forged_catalog = BabelCatalog(
            header_comment=self.header_comment,
            project=self.project.name,
            version=str(self.version)
        )
        
        for entry in self.templatemsg_set.all().order_by('id'):
            forged_catalog.add(entry.message, string=None, locations=entry.get_locations_set(), flags=entry.get_flags_set())
        
        return forged_catalog

    def __unicode__(self):
        return "{name} v{version}".format(name=self.project.name, version=self.version)

    class Meta:
        verbose_name = _('project version')
        verbose_name_plural = _('projects versions')

class TemplateMsg(models.Model):
    """
    Template catalog item, equivalent to a msg from a POT file
    """
    project_version = models.ForeignKey(ProjectVersion, verbose_name=_('project version'), blank=False)
    message = models.TextField(_('message id'), blank=False)
    locations = models.TextField(_('locations'))
    flags = models.TextField(_('flags'))

    def __unicode__(self):
        return self.message

    def get_truncated_message(self):
        """Return truncated message to avoid message too long"""
        if not self.message:
            return ''
        elif len(self.message) <= 50:
            return self.message
        return self.message[0:50]+' [...]'
    get_truncated_message.short_description = 'Message'

    def get_locations_set(self):
        """Return a set from stored locations in JSON"""
        return set([tuple(item) for item in json.loads(self.locations)])

    def get_flags_set(self):
        """Return a set from stored flags in JSON"""
        return set(json.loads(self.flags))

    class Meta:
        verbose_name = _('template message')
        verbose_name_plural = _('templates messages')

class Catalog(models.Model):
    """
    Language catalog for a project, the PO file equivalent
    """
    project_version = models.ForeignKey(ProjectVersion, verbose_name=_('project version'), blank=False)
    locale = models.CharField(_('locale'), max_length=50, blank=False)
    header_comment = models.TextField(_('header comment'))
    mime_headers = models.TextField(_('mime headers'))

    def __unicode__(self):
        return self.get_locale_name()

    def get_locale_name(self):
        l = Locale.parse(self.locale)
        return l.english_name

    def count_empty_translations(self):
        """Return a count of translations with an empty message"""
        return self.translationmsg_set.filter(message="").count()

    def count_fuzzy_translations(self):
        """Return a count of translations with message marked as fuzzy"""
        return self.translationmsg_set.filter(fuzzy=True).count()

    def get_babel_catalog(self):
        """
        Return a babel catalog suitable for a PO file
        """
        forged_catalog = BabelCatalog(
            locale=self.locale, 
            header_comment=self.header_comment,
            project=self.project_version.project.name,
            version=str(self.project_version.version)
        )
        
        for entry in self.translationmsg_set.all().order_by('id'):
            locations = [tuple(item) for item in json.loads(entry.template.locations)]
            forged_catalog.add(entry.template.message, string=entry.message, locations=locations, flags=entry.get_flags())
        
        return forged_catalog

    class Meta:
        verbose_name = _('catalog')
        verbose_name_plural = _('catalogs')
        permissions = (
            ('edit_messages', 'Edit messages'),
        )

class TranslationMsg(models.Model):
    """
    Translation message from a catalog
    """
    template = models.ForeignKey(TemplateMsg, verbose_name=_('row source'), blank=False)
    catalog = models.ForeignKey(Catalog, verbose_name=_('catalog'), blank=False)
    message = models.TextField(_('message'), blank=True)
    fuzzy = models.BooleanField(_('fuzzy'), default=False, blank=True)
    pluralizable = models.BooleanField(_('pluralizable'), default=False, blank=True)
    python_format = models.BooleanField(_('python_format'), default=False, blank=True)

    def __unicode__(self):
        return self.message

    def get_truncated_message(self):
        """Return truncated message to avoid message too long"""
        if not self.message:
            return ''
        elif len(self.message) <= 50:
            return self.message
        return self.message[0:50]+' [...]'
    get_truncated_message.short_description = 'Message'

    def get_flags(self):
        """Return a set of flags computed from some model attributes"""
        flags = []
        if self.fuzzy: flags.append('fuzzy')
        if self.python_format: flags.append('python-format')
        return set(flags)

    class Meta:
        verbose_name = _('translation message')
        verbose_name_plural = _('translations messages')



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Project'
        db.create_table(u'po_projects_project', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=75)),
        ))
        db.send_create_signal(u'po_projects', ['Project'])

        # Adding model 'ProjectVersion'
        db.create_table(u'po_projects_projectversion', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('project', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['po_projects.Project'])),
            ('version', self.gf('django.db.models.fields.SmallIntegerField')()),
            ('header_comment', self.gf('django.db.models.fields.TextField')()),
            ('mime_headers', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal(u'po_projects', ['ProjectVersion'])

        # Adding model 'TemplateMsg'
        db.create_table(u'po_projects_templatemsg', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('project_version', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['po_projects.ProjectVersion'])),
            ('message', self.gf('django.db.models.fields.TextField')()),
            ('locations', self.gf('django.db.models.fields.TextField')()),
            ('flags', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal(u'po_projects', ['TemplateMsg'])

        # Adding model 'Catalog'
        db.create_table(u'po_projects_catalog', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('project_version', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['po_projects.ProjectVersion'])),
            ('locale', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('header_comment', self.gf('django.db.models.fields.TextField')()),
            ('mime_headers', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal(u'po_projects', ['Catalog'])

        # Adding model 'TranslationMsg'
        db.create_table(u'po_projects_translationmsg', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('template', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['po_projects.TemplateMsg'])),
            ('catalog', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['po_projects.Catalog'])),
            ('message', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('fuzzy', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('pluralizable', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('python_format', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'po_projects', ['TranslationMsg'])


    def backwards(self, orm):
        # Deleting model 'Project'
        db.delete_table(u'po_projects_project')

        # Deleting model 'ProjectVersion'
        db.delete_table(u'po_projects_projectversion')

        # Deleting model 'TemplateMsg'
        db.delete_table(u'po_projects_templatemsg')

        # Deleting model 'Catalog'
        db.delete_table(u'po_projects_catalog')

        # Deleting model 'TranslationMsg'
        db.delete_table(u'po_projects_translationmsg')


    models = {
        u'po_projects.catalog': {
            'Meta': {'object_name': 'Catalog'},
            'header_comment': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'locale': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'mime_headers': ('django.db.models.fields.TextField', [], {}),
            'project_version': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['po_projects.ProjectVersion']"})
        },
        u'po_projects.project': {
            'Meta': {'object_name': 'Project'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '75'})
        },
        u'po_projects.projectversion': {
            'Meta': {'object_name': 'ProjectVersion'},
            'header_comment': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mime_headers': ('django.db.models.fields.TextField', [], {}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['po_projects.Project']"}),
            'version': ('django.db.models.fields.SmallIntegerField', [], {})
        },
        u'po_projects.templatemsg': {
            'Meta': {'object_name': 'TemplateMsg'},
            'flags': ('django.db.models.fields.TextField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'locations': ('django.db.models.fields.TextField', [], {}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'project_version': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['po_projects.ProjectVersion']"})
        },
        u'po_projects.translationmsg': {
            'Meta': {'object_name': 'TranslationMsg'},
            'catalog': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['po_projects.Catalog']"}),
            'fuzzy': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'pluralizable': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'python_format': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'template': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['po_projects.TemplateMsg']"})
        }
    }

    complete_apps = ['po_projects']


# -*- coding: utf-8 -*-
"""
Root url's map for application
"""
from django.conf.urls import *

from po_projects.views.projects import ProjectIndex, ProjectCreateView, ProjectDetails, ProjectExportView, ProjectUpdate
from po_projects.views.catalogs import CatalogDetails, CatalogMessagesExportView, CatalogMessagesFormView

urlpatterns = patterns('',
    url(r'^$', ProjectIndex.as_view(), name='project-index'),
    
    url(r'^create/$', ProjectCreateView.as_view(), name='project-create'),
    url(r'^(?P<slug>[-\w]+)/$', ProjectDetails.as_view(), name='project-details'),
    url(r'^(?P<slug>[-\w]+)/update/$', ProjectUpdate.as_view(), name='project-update'),
    url(r'^(?P<slug>[-\w]+)/download/$', ProjectExportView.as_view(), name='project-download'),
    
    url(r'^(?P<slug>[-\w]+)/(?P<locale>[-\w]+)/$', CatalogDetails.as_view(), name='catalog-details'),
    url(r'^(?P<slug>[-\w]+)/(?P<locale>[-\w]+)/edit/$', CatalogMessagesFormView.as_view(), name='catalog-messages-edit'),
    url(r'^(?P<slug>[-\w]+)/(?P<locale>[-\w]+)/download/$', CatalogMessagesExportView.as_view(), name='catalog-messages-download'),
)

"""
Urls map for API with Django REST Framework if installed

TODO: 

* project update from a POT file
* get project catalogs tarball

"""
try:
    from rest_framework.urlpatterns import format_suffix_patterns
except ImportError:
    pass
else:
    from po_projects.rest import views

    rest_urlpatterns = format_suffix_patterns(patterns('po_projects.rest.views',
        url(r'^rest/$', 'api_root'),
        url(r'^rest/projects/$', views.ProjectList.as_view(), name='api-project-list'),
        url(r'^rest/projects/current/(?P<slug>[-\w]+)/$', views.ProjectCurrentDetail.as_view(), name='api-project-detail'),
        url(r'^rest/projects/current/(?P<slug>[-\w]+)/tarball/$', views.ProjectArchive.as_view(), name='api-project-archive'),
        #url(r'^rest/projects/current/(?P<slug>[-\w]+)/pot-update/$', views.ProjectCurrentDetail.as_view(), name='api-project-pot-update'),
        url(r'^rest/projects/version/(?P<pk>\d+)/$', views.ProjectVersionDetail.as_view(), name='api-project-version-detail'),
    ))

    urlpatterns = rest_urlpatterns + urlpatterns



from setuptools import setup, find_packages

setup(
    name='PO-Projects',
    version=__import__('po_projects').__version__,
    description=__import__('po_projects').__doc__,
    long_description=open('README.rst').read(),
    author='David Thenon',
    author_email='dthenon@emencia.com',
    url='http://pypi.python.org/pypi/PO-Projects',
    license='MIT',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=[
        'Babel==1.3',
        'django-braces>=1.2.0,<1.4',
        'django-extra-views',
        'South==0.7.6',
        'autobreadcrumbs >= 0.9.1',
        'djangocodemirror==0.9.3',
        'django-crispy-forms >= 1.3.2',
        #'django-guardian >= 1.2',
        #'djangorestframework >= 2.3',
    ],
    include_package_data=True,
    zip_safe=False
)


"""
Serializers for REST entries
"""
from cStringIO import StringIO

from django.forms import widgets

from babel.messages.pofile import read_po

from rest_framework import serializers

from po_projects.models import Project, ProjectVersion, TemplateMsg, Catalog, TranslationMsg
from po_projects.utils import create_templatemsgs, create_new_version, update_catalogs

class VersionSerializer(serializers.ModelSerializer):
    """
    Serializer for ``ProjectVersion`` model
    """
    url = serializers.HyperlinkedIdentityField(
        view_name='po_projects:api-project-version-detail',
        lookup_field='pk'
    )
    class Meta:
        model = ProjectVersion
        fields = ('id','version','url')


class ProjectVersionSerializer(serializers.HyperlinkedModelSerializer):
    """
    Serializer for ``Project`` model for a specific version
    
    TODO: Add tarball_url link for the given version, not for the current one
    """
    url = serializers.HyperlinkedIdentityField(
        view_name='po_projects:api-project-version-detail',
        lookup_field='pk'
    )
    tarball_url = serializers.HyperlinkedIdentityField(
        view_name='po_projects:api-project-archive',
        lookup_field='slug'
    )
    
    class Meta:
        model = Project
        fields = ('url', 'id', 'slug', 'name', 'description')#, 'tarball_url' )


class ProjectCurrentSerializer(ProjectVersionSerializer):
    """
    Serializer for ``Project`` model for the current version (the last one)
    """
    url = serializers.HyperlinkedIdentityField(
        view_name='po_projects:api-project-detail',
        lookup_field='slug'
    )
    projectversion_set = VersionSerializer(
        many=True, read_only=True,
    )
    pot = serializers.CharField(
        widget=widgets.Textarea,
        write_only=True,
        required=False,
        help_text='(optionnal) Content for a valid POT file to update project strings to translate in its catalogs'
    )
    
    def validate_pot(self, attrs, source):
        """
        Validation for the given POT content
        """
        value = attrs[source]
        if value:
            try:
                template_file = StringIO()
                template_file.write(value.encode('UTF8'))
                template_file.seek(0)
                # Seems the validation from read_po is too much minimalistic
                # This does not really valid if the content is a real POT content
                self.uploaded_pot_file = read_po(template_file, ignore_obsolete=True)
            except:
                raise serializers.ValidationError("Your file does not seem to be a valid POT file")
        return attrs

    def save(self, **kwargs):
        """
        Override the save method to update template and catalog from the given POT
        """
        super(ProjectCurrentSerializer, self).save(**kwargs)
        
        if hasattr(self, 'uploaded_pot_file'):
            previous_version = self.object.get_current_version()
            current_version = create_new_version(self.object, previous_version.version+1, self.uploaded_pot_file)
            update_catalogs(self.object, previous_version, current_version)
            self.object.save()
         
        return self.object
    
    class Meta:
        model = Project
        fields = ('id', 'slug', 'name', 'description', 'url', 'tarball_url', 'pot', 'projectversion_set' )



# -*- coding: utf-8 -*-
"""
Dump tools
"""
from cStringIO import StringIO
import json, os, tarfile, time

from babel.messages.pofile import write_po
from babel import Locale
from babel.core import UnknownLocaleError, get_locale_identifier
from babel.messages.catalog import Catalog as BabelCatalog

from django.conf import settings

from po_projects.models import Project, Catalog

def po_project_export(project, project_version, archive_fileobj, catalog_filename):
    """
    Export all catalogs from a project into PO files with the good directory 
    structure
    """
    archive_files = []
    
    # Template catalog POT file
    template_file = StringIO()
    write_po(template_file, project_version.get_babel_template(), sort_by_file=False, ignore_obsolete=True, include_previous=False)
    template_file.seek(0)
    archive_files.append( (settings.POT_ARCHIVE_PATH.format(catalog_filename=catalog_filename), template_file) )
    
    # Catalog PO files
    for catalog in project_version.catalog_set.all():
        archived_path = settings.PO_ARCHIVE_PATH.format(locale=catalog.locale, catalog_filename=catalog_filename)
        # Open a new catalog
        babel_catalog = catalog.get_babel_catalog()
        # Write it to a buffer string
        catalog_file = StringIO()
        write_po(catalog_file, babel_catalog, sort_by_file=False, ignore_obsolete=True, include_previous=False)
        catalog_file.seek(0)
        
        archive_files.append( (archived_path, catalog_file) )
    
    # Open and fill tarball archive
    archive = tarfile.open("{0}.tar.gz".format(project.slug), mode="w:gz", fileobj=archive_fileobj)
    mtime = time.time()
    
    for name,content in archive_files:
        info = tarfile.TarInfo(name)
        info.size=len(content.getvalue())
        info.mtime = mtime
        archive.addfile(tarinfo=info, fileobj=content)
        #
        content.close()
    
    archive.close()
    
    return archive_fileobj

